<div class="kb-event-box">
  {{#eventImage}}
  <div class="kb-event-box-image">
    {{#user}}
    <a href="{{ user.url }}">
    {{/user}}
      <img src="{{eventImage}}" style="width: 48px; height: 48px;">
    {{#user}}
    </a>
    {{/user}}
  </div>
  {{/eventImage}}
  <div class="kb-event-box-details">
    {{#user}}
    <b><a href="{{user.url}}">{{ eventUser }}</a></b>
    {{/user}}
    {{^user}}
    <b>{{ eventUser }}</b>
    {{/user}}
    poured
    <a href="{{ drink.url }}">
      <span class="hmeasure">
        <span class="num">{{ drink.volume_ml }}</span>
        <span class="unit">mL</span>
      </span></a>
      {{#drink.shout}}
      <br/>
      <i>"{{drink.shout}}"</i>
      {{/drink.shout}}
    <br/>
  </div>
  <small><abbr class="timeago" title="{{ time }}">{{ time }}</abbr></small>
</div>

