{% extends "base.html" %}
{% load kegweblib %}
{% load crispy_forms_tags %}

{% block title %}Registration Closed{% endblock %}
{% block pagetitle %}Registration Closed{% endblock %}

{% block content %}
<p>
  Sorry, registration is closed.
</p>
{% endblock %}
