{# Bare-bones skeletal template. #}
{% load kegweblib %}
{% load humanize %}
{% if USE_FACEBOOK %}{% load facebook %}{% endif %}
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN"
    "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">
<head>
  <title>{% block title %}Kegbot{% endblock %}</title>

  {% block css %}
  <link rel="icon" type="image/png" href="{{ STATIC_URL}}images/kegbot-icon-72x72.png" />
  <link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}css/kegweb.css" media="screen"/>
  <link rel="stylesheet" type="text/css" href="{{ STATIC_URL }}fancybox/jquery.fancybox.css" media="screen" />
  {% endblock %}

  {% block js %}
  <script type="text/javascript" src="{{ STATIC_URL }}js/jquery-1.9.1.min.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}js/jquery.cookie.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}js/jquery.timeago.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}js/jquery.autounits.js"></script>

  <script type="text/javascript" src="{{ STATIC_URL }}bootstrap/js/bootstrap.min.js"></script>

  <script type="text/javascript" src="{{ STATIC_URL }}angular/angular.min.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}angular/angular-sanitize.min.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}angular/angular-resource.min.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}js/kegweb-angular-filters.js"></script>

  <script type="text/javascript" src="{{ STATIC_URL }}lib/underscore/underscore.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}lib/backbone/backbone.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}lib/mustache/mustache.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}lib/icanhaz/ICanHaz.js"></script>

  <script type="text/javascript" src="{{ STATIC_URL }}js/kb.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}highcharts/js/highcharts.js"></script>
  <script type="text/javascript" src="{{ STATIC_URL }}fancybox/jquery.fancybox.pack.js"></script>

  {% endblock js %}

  {% block kb-extrajs %}{%endblock%}

  {% if GOOGLE_ANALYTICS_ID %}
  <script type="text/javascript">

    var _gaq = _gaq || [];
    _gaq.push(['_setAccount', '{{GOOGLE_ANALYTICS_ID}}']);
    _gaq.push(['_trackPageview']);

    (function() {
      var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
      ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
      var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
    })();

  </script>
  {% endif %}
</head>

<body {% block bodyattrs %}{% endblock %}>
{% block body %}
{% endblock body %}
</body>

