{% extends "base.html" %}
{% load kegweblib %}
{% load bootstrap_pagination %}

{% block title %}Keg List | {{ block.super }}{% endblock %}
{% block pagetitle %}Keg List{% endblock %}

{% block content %}

{% for keg in kegs %}
  <div class="row">
    <div class="span10">
      {% include "kegweb/keg-snapshot.html" %}
    </div>
  </div>
{% endfor %}

{% bootstrap_paginate page_obj range=10 %}

{% endblock %}
