{% extends "setup_wizard/base.html" %}
{% load crispy_forms_tags %}

{% block content %}
<div class="jumbotron">

<h2>Kegbot Setup: Create or Import System</h2>
<p class="lead">
  Are we starting from scratch or importing an existing site?
</>
</div>

<div class="row-fluid">
  <div class="span12">
    {% crispy form %}
  </div>
</div>

{% endblock content %}
