{% extends "kegweb/drinkingsession_archive_base.html" %}

{% block title %}Sessions | {{ block.super }}{% endblock %}
{% block pagetitle %}Sessions{% endblock %}

{% block session-right %}
<ul>
  {% for d in date_list %}
  <li>
    <a href="{% url "kb-sessions-year" d.year %}">{{d.year}}</a>
  </li>
  {% endfor %}
</ul>
{% endblock %}
