{% extends "kegadmin/base.html" %}
{% load kegweblib %}
{% load crispy_forms_tags %}

{% block title %}Kegbot Admin: Edit Beer Type | {{ block.super }}{% endblock %}
{% block pagetitle %}Kegbot Admin: Edit Beer Type {% endblock %}

{% block kegadmin-main %}

<h2>Edit Beer Type</h2>

{% if beer_type.image %}
<div class="row-fluid">
<div class="span10">
<p><img src="{{beer_type.image.resized.url}}"/></p>
</div>
</div>
{% endif %}
    
{% crispy form %}

{% endblock %}
