{% extends "base.html" %}
{% block content %}
<div class="row">
  <div class="span8"> <!-- left col -->
  {% block col-1 %}{% endblock %}
  </div> <!-- end center col -->
  <div class="span4"> <!-- right col -->
  {% block col-2 %}{% endblock %}
  </div> <!-- end right col -->
</div>
{% endblock %}
