{% extends "base.html" %}
{% load kegweblib %}

{% block content %}

<div class="row-fluid">
  <div class="span2">
    <ul class="nav nav-list">
      {% include "kegadmin/nav-items.html" %}
    </ul>
  </div>

  <div class="span10">
    {% block kegadmin-main %}{% endblock %}
  </div>
</div>

{% endblock %}
