{% extends "base.html" %}
{% load kegweblib %}

{% block title %}Access Denied: Members Only | {{ block.super }}{% endblock %}
{% block pagetitle %}Access Denied{% endblock %}

{% block content %}
<p>
You need to log in as a registered drinker in order to see this content.
</p>
{% endblock %}
