{% extends "kegweb/drinkingsession_archive_base.html" %}
{% load tz %}

{% block title %}Sessions: {{month|date:"F Y"}} | {{ block.super }}{% endblock %}
{% block pagetitle %}Sessions: {{month|date:"F Y"}}{% endblock %}

{% block session-right %}
<ul>
  <li>
    <a href="{% url "kb-sessions" %}">All Sessions</a>
    <ul>
      <li>
        <a href="{% url "kb-sessions-year" month.year %}">{{month|date:"Y"}}</a>
        <ul>
          <li>
            <a href="{% url "kb-sessions-month" month.year month.month %}"><b>{{month|date:"F"}}</b></a>
            <ul>
              {% for d in date_list %}
              <li>
                <a href="{% url "kb-sessions-day" d.year d.month d.day %}">{{ d|utc|date:"F d" }}</a>
              </li>
              {% endfor %}
            </ul>
          </li>
        </ul>
      </li>
    </ul>
  </li>
</ul>
{% endblock %}
