{% extends "kegweb/drinkingsession_archive_base.html" %}

{% block title %}Sessions: {{day|date:"F d, Y"}} | {{ block.super }}{% endblock %}
{% block pagetitle %}Sessions: {{day|date:"F d, Y"}}{% endblock %}

{% block session-right-hide %}{% endblock %}
