{% extends "page-twocol.html" %}
{% load kegweblib %}
{% load facebook %}

{% block title %}link your facebook account{% endblock %}

{% block col-1 %}
<div class="kb-contentbox-head">
  link your facebook account
</div>
<div class="kb-contentbox">
  <p>
    before you can announce your beers on facebook, you need to link your
    facebook account.
  </p>
  <p>
    <b>note:</b> you will be able to configure settings after linking; the
    default is to <i>not</i> post anything to facebook.
  </p>
  <p>
    {% facebook_button %}
  </p>
</div>
{% endblock %}
