{% extends "account/base.html" %}
{% load kegweblib %}
{% load crispy_forms_tags %}

{% block title %}Account: Change Password | {{ block.super }}{% endblock %}
{% block pagetitle %}Account: Change Password{% endblock %}

{% block kb-account-main %}
<h2>Change Password</h2>
<div class="well">
  <form method="post" action="">
  {% csrf_token %}
  {{ form|crispy }}
  <div class="actions">
    <button type="submit" class="btn btn-primary">Save Changes</button>
  </div>
  </form>
</div>

{% endblock %}
