{% load kegweblib %}

<li class="nav-header">Kegbot Admin</a>
{% navitem "kegadmin-dashboard" "Dashboard" exact %}

<li class="nav-header">General</a>
{% navitem "kegadmin-main" "General Settings" %}
{% navitem "kegadmin-logs" "Logs" %}

<li class="nav-header">System</li>
{% navitem "kegadmin-taps" "Taps" %}
{% navitem "kegadmin-drinks" "Drinks" %}
{% navitem "kegadmin-users" "Users" %}
{% navitem "kegadmin-tokens" "Tokens" %}

<li class="nav-header">Beer DB</li>
{% navitem "kegadmin-brewers" "Beer Brewers" %}
{% navitem "kegadmin-beer-styles" "Beer Styles" %}
{% navitem "kegadmin-beer-types" "Beer Types" %}

{% if PLUGINS %}
<li class="nav-header">Plugins</li>
{% for plugin in PLUGINS.values %}
{% url "kegadmin-plugin-settings" plugin_name=plugin.get_short_name as settings_url %}
{% navitem settings_url plugin.get_name %}
{% endfor %}
{% endif %}

<li class="divider"></li>
<li class="nav-header">Other</li>
{% if HAVE_ADMIN %}
<li><a href='{% url "admin:index" %}' target="_blank">Database Admin &raquo;</a></li>
{% endif %}
<li><a href='https://github.com/Kegbot/kegbot/issues' target="_blank">Report a Bug &raquo;</a></li>
